clear 
smt_init
load -mat step_index.gd
oGd.nSources = 30;
oGd.alphaIn = 0.5;
positions = distribute(oGd);

vLambda = 1.5:0.05:2;
steps = length(vLambda);

mNeff = []; % matrix for effective indices
mErr = []; % matrix for continuity condition error
tol = 1e-6; % tolerance
maxN = 7; % Max. Number of points in a monotonic interval
bPlot = 0; % Show progress
complex = 0;
minNeffI = 0; % irrelevant for real effective index
method = 'direct'; % Eigenvalue determination method.
minNeff = 1.505;
maxNeff = 1.75;
maxErr = 0.01; % Maximum acceptable error
tic
for ind = 1:steps 
    lambda = vLambda(ind);
    k0 = 2*pi/lambda;
    if bPlot
        figure(1)
    end
    [vNeff, vErr] = find_modes(oGd, positions, k0, minNeff, ...
        maxNeff, tol, bPlot, maxN, maxErr, complex, minNeffI, method);
    if vNeff == -1
        break;
    end
    figure(2)
    vNeff = vNeff(:);
    vErr = vErr(:);
    if isempty(mNeff)
        mNeff = [mNeff vNeff];
        mErr = [mErr vErr];
    else
        PrevBeta = mNeff(:,end);
        % deal with the possibility of modes going in or out of the 
        % range of Neff and lambda, by filling their places in the 
        % mNeff with NaN
        PrevNaN = ones(length(vNeff)-length(PrevBeta), size(mNeff,2))*NaN;
        CurrNaN = ones(length(PrevBeta)-length(vNeff), 1)*NaN;
        if ~isempty(PrevNaN)
            mNeff = [mNeff; PrevNaN];
            mErr = [mErr; PrevNaN];
        end
        if ~isempty(CurrNaN)
            vNeff = [vNeff; CurrNaN];
            vErr = [vErr; CurrNaN];
        end
        mNeff = [mNeff vNeff];
        mErr = [mErr vErr];
    end        
    fraction_done = ind/steps;
    fprintf(1, 'percent done: %d\n', round(fraction_done*100))
    t = toc;
    fprintf(1,'minutes to go: %6.2f\n', (1-fraction_done)/fraction_done*t/60)
    plot(2*pi./vLambda(1:ind), mNeff', 'b.')
    xlabel('$k_0$')
    ylabel('$n_\mathrm{eff}$')
end