function draw_geom(G, varargin)
% DRAW_GEOM(oGd, ...)
% Draws the geometry described by oGd. Scales the axes to show the whole
% geometry. Relables x and y axes.
% DRAW_GEOM(oGd, 'noaxischange') does not scale the axes.
% DRAW_GEOM(oGd, 'nolabel') does not relabel the axes.
% DRAW_GEOM(oGd, 'nothing') does neither.
bAxesCh = 1;
bLabel = 1;
if nargin == 2
    if strcmp(varargin{1}, 'nolabel')
        bLabel = 0;
    end
    if strcmp(varargin{1}, 'noaxischange')
        bAxesCh = 0;
    end
    if strcmp(varargin{1}, 'nothing')
        bAxesCh = 0;
        bLabel = 0;
    end
end
bNorm = 0;
if nargin == 3
    if strcmp(varargin{1}, 'normalize')
        lambda = varargin{2};
        bNorm = 1;
        bAxesCh = 0;
    end
end
if isfield(G, 'bInterface')
    return;
end
nCurves = G.nSd-1;
n = 1000;
if bAxesCh
    hold off
else
    hold on
end
for iCurve = 1:nCurves
    curve = G.curveArray(iCurve);
    cs = curve.cs;
    s = linspace(cs.breaks(1), cs.breaks(end) ,n);
    mF = ppval(cs, s);
    vSx = mF(1,:);
    vSy = mF(2,:);
    vSx = vSx+curve.xc;
    vSy = vSy+curve.yc;
    if bNorm
        vSx = vSx/lambda;
        vSy = vSy/lambda;
    end
    plot(vSx, vSy,'b', 'linewidth', 1)
    hold on
%     if bLabel
%         fixfonts(text(curve.xc, curve.yc, '\epsilon_{r2}'), fs/1.2); 
%     end
end
if bNorm
    xlabel('$x/\lambda$')
    ylabel('$x/\lambda$')
else
    xlabel('x[m]')
    ylabel('y[m]')
end
set(gca, 'ydir', 'normal')
if bAxesCh
    axis equal
    ax = axis;
    axis(expand_axis(ax,1.2));
end
hold off