% green_fun should return the tangential fields at the testing points of
% electric and magnetic sources placed at 'sources'. If varargin is not
% empty it returns the component defined by the penultimate argument. The
% last argument is the source type in this case.
function varargout = green_fun(testing, sources, Y, kc, beta, oGd, varargin)
k_2 = beta^2 + kc^2;
Z0 = -1/Y*k_2;
vGreen = sources.vGreen;
ang = 2*pi/length(vGreen);
bFini = any(imag(vGreen));
if ~isempty(varargin)
    sComp = varargin{1};
    x = testing.vX(:);
    y = testing.vY(:);
    x_s = sources.x;
    y_s = sources.y;
    fieldJ_sum = zeros(size(x));
    fieldM_sum = zeros(size(x));
    for iReflect = 1:length(vGreen)
        rho = sqrt((x - x_s).^2 + (y - y_s).^2);
        switch lower(sComp)
            case 'ex'
                Erho = 1/(4*Y)*beta*kc*hankel2(1, kc*rho);
                Hphi = Erho*Y/beta/j;
                fieldJ = Erho.*(x-x_s)./rho;
                fieldM = Hphi.*(y-y_s)./rho;
            case 'ey'
                Erho = 1/(4*Y)*beta*kc*hankel2(1, kc*rho);
                Hphi = Erho*Y/beta/j;
                fieldJ = Erho.*(y-y_s)./rho;
                fieldM = -Hphi.*(x-x_s)./rho;
            case 'ez'
                fieldJ = 1/(Y*4*j)*kc^2*hankel2(0,kc*rho);
                fieldM = 0;
            case 'hx'
                Hphi = 1/4j*kc*hankel2(1, kc*rho);
                Hrho = Hphi*j*beta/Z0;
                fieldJ = -Hphi.*(y-y_s)./rho;
                fieldM = Hrho.*(x-x_s)./rho;
            case 'hy'
                Hphi = 1/4j*kc*hankel2(1, kc*rho);
                Hrho = Hphi*j*beta/Z0;
                fieldJ = Hphi.*(x-x_s)./rho;
                fieldM = Hrho.*(y-y_s)./rho;
            case 'hz'
                fieldJ = 0;
                fieldM = 1/4j*kc^2*hankel2(0,kc*rho)/Z0;
        end
        fieldJ_sum = fieldJ_sum + fieldJ*vGreen(iReflect);
        fieldM_sum = fieldM_sum + fieldM*vGreen(iReflect)*(2*bFini-1)^(iReflect-1);
        if iReflect ~= length(vGreen)
            [x_s, y_s] = reflect(x_s, y_s, ang, iReflect, oGd.offsetX, oGd.offsetY);
        end
    end
    [rows, cols] = size(testing.vX);
    fieldJ_sum = reshape(fieldJ_sum, rows, cols);
    fieldM_sum = reshape(fieldM_sum, rows, cols);
    [varargout{1:2}] = deal(fieldJ_sum, fieldM_sum);
    return
end
rows = length(testing.vX);
cols = length(sources.vX);
ang = 2*pi/length(vGreen);
[Fe, Ee, Be, Fm, Em, Bm] = deal(zeros(rows, cols));
bFini = any(imag(vGreen));
for iReflect = 1:length(vGreen)
    [iFe, iEe, iBe] = e_sources(testing, sources, Y, kc, beta);
    iFe = iFe*vGreen(iReflect);
    iEe = iEe*vGreen(iReflect);
    iBe = iBe*vGreen(iReflect);
    iFm = iFe*(2*bFini-1)^(iReflect-1);
    iEm = iEe*(2*bFini-1)^(iReflect-1);
    iBm = iBe*(2*bFini-1)^(iReflect-1);
    Fe = Fe + iFe;
    Ee = Ee + iEe;
    Be = Be + iBe;
    Fm = Fm + iFm;
    Em = Em + iEm;
    Bm = Bm + iBm;
    if iReflect ~= length(vGreen)
        [sources.vX, sources.vY] = reflect(sources.vX, sources.vY, ang, iReflect,...
            oGd.offsetX, oGd.offsetY);
    end
end
mZeros = zeros(size(Fm));
ETA_sd2 = Z0/Y;
TE_Et = -Fm;
TE_Ez = mZeros;
TE_Ht = Em/ETA_sd2;
TE_Hz = Bm/ETA_sd2;
TM_Et = Ee;
TM_Ez = Be;
TM_Ht = Fe;
TM_Hz = mZeros;
[varargout{1:8}] = deal(TE_Et, TE_Ez, TE_Ht, TE_Hz, TM_Et, TM_Ez, TM_Ht, TM_Hz);
