function ogd2dxf(oGd, fname)
% OGD2DXF(oGd, filename)
% Exports the geometry in oGd to a DXF file named filename.
% Note that the '.DXF' extension is not added automatically, 
% so it should be included in filename. 
[fid, err] = DXF_start(fname,1);
if err
    error(['Error opening output file: ' fname]);
    return
end
curves = oGd.curveArray;
linecolour = 7;
linetype = 1;
for iCurve = 1:length(curves)
    switch curves(iCurve).tag
        case 'circle'
            DXF_circle(fid, curves(iCurve).xc, curves(iCurve).yc, curves(iCurve).param(1), linecolour, linetype);
        case 'ellipse'
            mx = curves(iCurve).param(1);
            my = 0;
            mr = curves(iCurve).param(2);
            DXF_ellipse(fid, curves(iCurve).xc, curves(iCurve).yc, ...
                mx, my, mr, 0, 2*pi, linecolour, linetype);
        otherwise
            Bsp = fn2fm(curves(iCurve).cs,'B-');
            knot = Bsp.knots;
            x = Bsp.coefs(1,:)+curves(iCurve).xc;
            y = Bsp.coefs(2,:)+curves(iCurve).yc;
            nknot = length(knot);
            ncontrol = length(x);
            DXF_spline(fid,nknot,ncontrol,knot,x,y,linecolour,linetype);
    end
end
DXF_end(fid)