function [vErr, varargout] = smt_solve_filtered(Neff, oGd, pos, k0, vSol_filter, min_err, varargin)
% error = SMT_SOLVE(Neff, oGd, positions, k0)
% Solves the geometry defined in oGd.
% Sources and testing points are distributed according to positions.
% Free-space wave number: k0.
% Effective index: Neff, may be a vector
% Returns the error in continuity conditions.
% [error, solution] = SMT_SOLVE(Neff, oGd, pos, k0)
% Returns the vector of source amplitudes as well.
% If Neff is a vector, solution is a matrix. Each column of the matrix
% corresponds to one entry of the Neff vector.
% If Neff is complex, the function must be called with one more argument:
% [error, solution] = SMT_SOLVE(Neff, oGd, pos, k0, isImproper).
% isImproper determines whether the field obeys the radiation condition.
persistent guess
global SOLVE_CNT
SOLVE_CNT = SOLVE_CNT+1;

nTries = 4;
if length(varargin) == 2
    Neff = Neff(1) + j*Neff(2);
%          figure(1)
%      hold on
%      plot(real(Neff), imag(Neff),'k.')
%      drawnow
end
vBeta = Neff*k0;
vErr = zeros(1,length(vBeta));
mSol = zeros(2*sum([pos.sources.n]), length(vBeta));
if ~isreal(Neff)
    isImproper = varargin{1};
else
    isImproper = 0; % if Neff is real these values do not make any difference 
end
for ind = 1:length(vBeta)
    beta = vBeta(ind);
    [mZ, mBNegate] = imp_matrix(oGd, pos, k0, beta, isImproper);
    if (length(guess) ~= size(mZ, 2))
        guess = [];
    end
    err = -1;
    iTry = 1;
    while (err == -1) && (iTry < nTries)
        if iTry > 1
            guess = [];
            disp('Arnoldi convergence failure')
        end
        [err, guess] = non_trivial_filtered(mZ, mBNegate, vSol_filter, min_err, guess);
        iTry = iTry + 1;
    end
    vErr(ind) = err;
%     hold on, plot(Neff,err,'.')
%     Neff, err
%     pause
    if nargout == 2
        mSol(:,ind) = guess(:);
    end
end
if nargout == 2
    varargout{1} = mSol;
end