function varargout = smtgui(varargin)
% Source-Model Technique Graphical User Interface

% for compiler:
%#function silver 
%#function gold
%#function copper
%#function aluminum
%#function sellmeier
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @smtgui_OpeningFcn, ...
    'gui_OutputFcn',  @smtgui_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

% --- Executes just before smtgui is made visible.
end

function smtgui_OpeningFcn(hObject, eventdata, handles, varargin)
global EPSILON0 MU0 C DELTA ETA bCancel
global FontSize defPointsX defPointsY
C = 2.99792458e8; %[m/sec]
EPSILON0 = 1e7/(4*pi*C^2); %[F/m]
MU0 = 4*pi*1e-7; %[H/m]
ETA = sqrt(MU0/EPSILON0);
DELTA = 1e-5;
FontSize = 13;
bCancel = 0;
defPointsX = 150;
defPointsY = 150;

handles.defaults = get(0, 'default');
guidata(hObject, handles)
set(0, 'defaultAxesFontSize', FontSize);
set(0, 'defaultAxesFontName', 'Times New Roman');
set(0, 'defaultTextInterpreter', 'latex');
toggle_save(handles);
uiNew_Callback(hObject, eventdata, handles)
add_to_text(hObject, handles, '>> smt_init');

% UIWAIT makes smtgui wait for user response (see UIRESUME)
% uiwait(handles.uiSmtgui);


% --- Outputs from this function are returned to the command line.
end

function varargout = smtgui_OutputFcn(hObject, eventdata, handles)
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;

% --- Executes on button press in uiFindModes.
end

function uiFindModes_Callback(hObject, eventdata, handles)
global sResults sFirstResults FontSize
% hObject    handle to uiFindModes (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
if ~check_oGd(handles)
    return
end
pos_axes_full = [0.088, 0.31, 0.856, 0.555];
pos_axes_half = [0.088, 0.31, 0.344, 0.555];
pos_axes2_full = [0.574, 0.31, 0.37, 0.555];
pos_axes2_half = [0.574, 0.64, 0.37, 0.239];
sFormat = ['%1.' num2str(2-round(log10(handles.tol))) 'g'];
data = gui_findbeta('min_lambda', num2str(handles.min_lambda, sFormat), 'max_lambda', num2str(handles.max_lambda, sFormat),...
    'n_lambda', num2str(handles.n_lambda),'min_beta1', num2str(handles.min_beta1, sFormat),...
    'min_beta2', num2str(handles.min_beta2, sFormat), 'max_beta1', num2str(handles.max_beta1, sFormat),...
    'max_beta2', num2str(handles.max_beta2, sFormat), 'tol', num2str(handles.tol), ...
    'absMinErr', num2str(handles.absMinErr), 'bPlot', num2str(handles.bPlot),...
    'nSampMono', num2str(handles.nSampMono), 'complex', num2str(handles.complex),...
    'maxNeffI', num2str(handles.maxNeffI), 'method', handles.method);
if data.status == 0
    if isSaveOff(handles)
        toggle_save(handles)
    end
    handles.maxNeffI = data.maxNeffI;
    handles.complex = data.complex;
    handles.nSampMono = data.nSamp;
    handles.max_beta1 = data.max_beta_val1;
    handles.max_beta2 = data.max_beta_val2;
    handles.min_beta1 = data.min_beta_val1;
    handles.min_beta2 = data.min_beta_val2;
    handles.min_lambda = data.min_lambda_val;
    handles.max_lambda = data.max_lambda_val;
    handles.n_lambda = data.nLambda_val;
    handles.bPlot = data.bPlot;
    handles.tol = data.tol;
    handles.absMinErr = data.minErr;
    handles.method = data.method;
    if handles.n_lambda == 1
        vLambda = handles.min_lambda;
    else
        set(handles.uiAxes, 'position', pos_axes_half);
        vLambda = linspace(handles.min_lambda, handles.max_lambda, handles.n_lambda);
        %vLambda = 1./linspace(1/handles.max_lambda, 1/handles.min_lambda, handles.n_lambda);
    end
    guidata(hObject, handles);
    if ~isfield(handles, 'pos')
        make_geom(hObject, handles);
        handles = guidata(hObject);
    end
    oGd = handles.oGd;
    pos = handles.pos;
    if handles.strClass(1) ~= 'n'
        pos = cut_pos(oGd, pos, handles.strClass);
    end
    tol = data.tol;
    absMinErr = data.minErr;
    nSampMono = data.nSamp;
    bPlot = data.bPlot;
    Complex = data.complex*(data.complex>0);
    maxNeffI = data.maxNeffI;
    sMethod = data.method;
    bDisp = (length(vLambda)>1);
    if bPlot & bDisp
        bPlot = 2;
    end
    mBeta = [];
    mErr = [];
    hWaitbar = [];
    for iLambda = 1:length(vLambda)
        lambda = vLambda(iLambda);
        if length(vLambda) == 1
            min_beta = handles.min_beta1;
            max_beta = handles.max_beta1;
        else
            x0 = min(vLambda);
            x1 = max(vLambda);
            y0 = handles.min_beta1;
            y1 = handles.min_beta2;
            min_beta = y0 + (y1-y0)/(x1-x0)*(lambda-x0);
            y0 = handles.max_beta1;
            y1 = handles.max_beta2;
            max_beta = y0 + (y1-y0)/(x1-x0)*(lambda-x0);
        end
        k0 = 2*pi/lambda;
        s1 = '>> [Neff, error] = find_modes(oGd, positions, 2*pi/lambda, minNeff, maxNeff, tol, bShow, maxN, maxErr, complex, minNeffI, method);';
        s2 = sprintf('   %% lambda = %g, min_Neff = %g, max_Neff = %g, tol = %g, bShow = %1.0f, maxN = %d, maxErr = %g, complex = %g, minNeffI = %g, method = ''%s''',...
            lambda, min_beta, max_beta, tol, bPlot, nSampMono, absMinErr, Complex, maxNeffI, sMethod);
        add_to_text(hObject, handles, s1, s2);
        tic
        [vBeta, vErr, hWaitbar] = find_modes(oGd, pos, k0, min_beta,...
            max_beta, tol, bPlot, nSampMono, absMinErr, Complex, maxNeffI, sMethod, handles.uiAxes, hWaitbar);
        toc
        if vBeta == -1
            figure(handles.uiSmtgui);
            guidata(hObject, handles);
            reset_axes
            reset_panels
            return
        end
        if bDisp & (iLambda == 1)
            sResults = sFirstResults;
        end
        if isempty(vBeta) & ~bDisp
            uiwait(msgbox('No modes found. Try increasing N_{max}', 'smtgui', 'warn', 'modal'));
            figure(handles.uiSmtgui);
            guidata(hObject, handles);
            reset_axes
            reset_panels
            return
        end
        vBeta = vBeta(:);
        vErr = vErr(:);
        if isempty(vBeta)
            continue
        end
        if isempty(mBeta)
            mBeta = [mBeta vBeta];
            mErr = [mErr vErr];
        else
            PrevBeta = mBeta(:,end);
            PrevNaN = ones(length(vBeta)-length(PrevBeta), size(mBeta,2))*(NaN+j*NaN);
            CurrNaN = ones(length(PrevBeta)-length(vBeta), 1)*(NaN+j*NaN);
            if ~isempty(PrevNaN)
                mBeta = [mBeta; PrevNaN];
            end
            if ~isempty(CurrNaN)
                vBeta = [vBeta; CurrNaN];
            end
            mBeta = [mBeta vBeta];
        end
        if isempty(mBeta)
            continue
        end
        if bDisp
            hAx = handles.uiAxes2;
            cla(hAx)
            set(hAx, 'visible', 'on');
            [rows, cols] = size(mBeta);
            if isreal(mBeta(~isnan(mBeta)))
                set(hAx, 'position', pos_axes2_full)
                plot(hAx, 2*pi./vLambda(1:cols), real(mBeta), 'b.')
                ylabel(hAx, '$n_\mathrm{eff}$')
            else
                hAx2 = handles.uiAxes3;
                set(hAx, 'position', pos_axes2_half)
                set(hAx2, 'visible', 'on');
                plot(hAx, 2*pi./vLambda(1:cols), real(mBeta), '.')
                plot(hAx2, 2*pi./vLambda(1:cols), imag(mBeta), '.');
                ylabel(hAx2, '$\mathrm{Im}(n_\mathrm{eff})$')
                ylabel(hAx, '$\mathrm{Re}(n_\mathrm{eff})$')
                xlabel(hAx2, '$k_0 [1/\mathrm{m}]$')
            end
            xlabel(hAx, '$k_0 [1/\mathrm{m}]$')
            drawnow
        end
        data = show_results('visible', 'off', 'Toggle', handles.uiShowRes,...
           'Beta', num2str(vBeta,'%0.20g'), 'Err', num2str(vErr), 'Tol', num2str(tol),...
           'lambda', num2str(lambda,'%0.6g'), 'complex', num2str(Complex));
    end
    reset_panels
    if isempty(mBeta)
        uiwait(msgbox('No modes found. Try increasing N_{max}.', 'smtgui', 'warn', 'modal'));
        guidata(hObject, handles);
        figure(handles.uiSmtgui)
        reset_axes
        axes(handles.uiAxes)
        draw_geom(oGd);
        return
    end
    if bDisp
        hFig = figure;
        set(hFig, 'name', 'Dispersion Curves')
        if ispc
            set(hFig, 'color', [236 233 216]/256);
        end
        figure(hFig);
        if isreal(mBeta(~isnan(mBeta)))
            plot(2*pi./vLambda(1:cols), real(mBeta), 'b.')
            ylabel('$n_\mathrm{eff}$')
            xlabel('$k_0 [1/\mathrm{m}]$')
        else
            subplot(211)
            plot(2*pi./vLambda(1:cols), real(mBeta), '.')
            ylabel('$\mathrm{Re}(n_\mathrm{eff})$')
            xlabel('$k_0 [1/\mathrm{m}]$')
            subplot(212)
            plot(2*pi./vLambda(1:cols), imag(mBeta), '.');
            ylabel('$\mathrm{Im}(n_\mathrm{eff})$')
            xlabel('$k_0 [1/\mathrm{m}]$')
        end

        drawnow
    end
    figure(handles.uiSmtgui);
    reset_axes
    handles.hResults = data.figure1;
    set(handles.uiShowRes, 'Value', 1);
    if bDisp
        figure(hFig);
    else
        figure(data.figure1)
    end
    guidata(hObject, handles);
end

    function reset_axes
        set(handles.uiAxes, 'position', pos_axes_full);
        set(handles.uiAxes2, 'visible', 'off')
        set(handles.uiAxes3, 'visible', 'off')
        cla(handles.uiAxes)
        cla(handles.uiAxes2);
        cla(handles.uiAxes3);
        axes(handles.uiAxes)
        draw_geom(oGd);
    end

    function reset_panels
        delete(hWaitbar)
        set(handles.text, 'visible', 'on')
        set(handles.uiPanel, 'visible', 'off')
        set(findobj('name', 'Effective Indices'), 'visible', 'on')
    end
end

% --------------------------------------------------------------------

function FileMenu_Callback(hObject, eventdata, handles)
% hObject    handle to FileMenu (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
end

function load_geom_Callback(hObject, eventdata, handles)
global sResults
% hObject    handle to OpenMenuItem (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
file = -1;
while 1
    [file, pathname, filter] = uigetfile( {'*.gd','Geometry Description Files (*.gd)';
        '*.se', 'Session File (*.se)'}, 'Open Session/Geometry', '');
    if isequal(file, 0)
        break;
    end
    if strcmpi(file(end-2:end), '.se') % session file
        s = load('-mat', [pathname file], 'oSettings', 'sResults');
        if ~isfield(s, 'oSettings')
            h = errordlg('Settings file does not include an ''oSettings'' variable', 'Error');
            set(h, 'windowstyle', 'modal');
            waitfor(h);
            continue
        else
            sResults = s.sResults;
            oSettings = s.oSettings;
            handles.sesPath = pathname;
            handles.sesFile = file;
            fields = fieldnames(oSettings);
            for ind = 1:length(fields)
                field = fields{ind};
                handles.(field) = oSettings.(field);
            end
            set(handles.uiShowRes, 'Value', 1);
            uiShowRes_Callback(handles.uiShowRes, eventdata, handles)
        end
        pathname = handles.gdPath;
        file = handles.gdFile;
        if ispc
            file_separator = '\';
        else
            file_separator = '/';
        end
        if isempty(dir([pathname file]))
            if isempty(dir([pwd file_separator file]))
                h = errordlg(['Geometry description file ' pathname file ' not found'], 'Error');
                set(h, 'windowstyle', 'modal');
                waitfor(h);
                return
            end
            pathname = [pwd file_separator];
            handles.gdPath = pathname;
        end
    end
    s = load('-mat', [pathname file], 'oGd');
    if isSaveOff(handles)
        toggle_save(handles)
    end
    if ~isfield(s, 'oGd')
        h = errordlg('Geometry file does not include an ''oGd'' variable', 'Error');
        set(h, 'windowstyle', 'modal');
        waitfor(h);
        continue
    else
        if isfield(handles, 'pos')
            handles  = rmfield(handles, 'pos');
        end
        set(handles.text, 'String', strvcat('Command-line log', ['>> load -mat ' file], ['>> draw_geom(oGd)']))
        oGd = s.oGd;
        handles.gdFile = file;
        handles.gdPath = pathname;
        set(handles.uiSmtgui, 'name', ['Smtgui - ' handles.sesFile ' (Geometry: ' file ')']);
        f = findobj('name', 'Set Parameters');
        delete(f)
        f = findobj('name', 'Mode Plot');
        delete(f)
        figure(handles.uiSmtgui)
        axes(handles.uiAxes);
        cla
        draw_geom(oGd);
        set(handles.uiAxes, 'visible', 'on');
        handles.oGd = oGd;
        if get(handles.uiShowHide, 'Value')
            make_geom(hObject, handles);
            handles = guidata(hObject);
            pos = handles.pos;
            show_pos(oGd, pos);
        end

        ax = handles.uiAxes;
        xlim = get(ax, 'xlim');
        ylim = get(ax, 'ylim');
        handles.xmin = xlim(1);
        handles.xmax = xlim(end);
        handles.ymin = ylim(1);
        handles.ymax = ylim(end);
        guidata(hObject, handles);
        break
    end
end

% --------------------------------------------------------------------
end

function CloseMenuItem_Callback(hObject, eventdata, handles)
% hObject    handle to CloseMenuItem (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
bOk = check_before_close(handles);
if bOk
    f = findobj('name', 'Effective Indices');
    delete(f)
    f = findobj('name', 'Set Parameters');
    delete(f)
    f = findobj('name', 'Mode Plot');
    delete(f)
    deleteall(handles.uiSmtgui)
    set(0, handles.defaults)
end


% --- Executes during object creation, after setting all properties.
end

function text_CreateFcn(hObject, eventdata, handles)
% hObject    handle to text (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end


end

function text_Callback(hObject, eventdata, handles)
% hObject    handle to text (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of text as text
%        str2double(get(hObject,'String')) returns contents of text as a
%        double

% --- Executes during object creation, after setting all properties.
end

function axes_CreateFcn(hObject, eventdata, handles)
% hObject    handle to axes (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: place code in OpeningFcn to populate axes


% --------------------------------------------------------------------
end

function uiSave_res_Callback(hObject, eventdata, handles)
% hObject    handle to uiSave_res (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
end

function uiSave_log_Callback(hObject, eventdata, handles)
% hObject    handle to uiSave_log (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --- Executes on button press in pushbutton3.
end

function pushbutton3_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton3 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --- Executes on button press in uiPlotFields.
end

function uiPlotFields_Callback(hObject, eventdata, handles)
global sResults
% hObject    handle to uiPlotFields (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
f = findobj('name', 'Set Parameters');
if ~isempty(f)
    figure(f)
    return
end
if ~check_oGd(handles)
    return
end
if ~isfield(handles, 'pos')
    make_geom(hObject, handles);
    handles = guidata(hObject);
end
pos = handles.pos;
add_to_text(hObject, handles, '>> % Solution is calculated at a specific point on the dispersion curve by:',...
    '>> [L, vSolution] = smt_solve(Neff, oGd, positions, 2*pi/lambda, method);',...
    '>> % L is the error, vSolution the source amplitudes, Neff - effective index, lambda - wavelength',...
    '>> % Field is calculated by:',...
    '>> mF = CalcField(vSolution, positions, oGd, 2*pi/lambda, Neff*2*pi/lambda, strComp, strClass, vX, vY);',...
    '>> % strComp is the field component: ''Ez'', etc.',...
    '>> % strClass is the symmetry class: ''p5'', ''f3'', etc.',...
    '>> % vectors vX and vY: X and Y values for evaluation');
data = gui_field_plot('xmin', num2str(handles.xmin), 'xmax', num2str(handles.xmax),...
    'ymin', num2str(handles.ymin), 'ymax', num2str(handles.ymax),...
    'scomp', handles.sComp, 'pointsx', num2str(handles.pointsx), 'pointsy', num2str(handles.pointsy),...
    'recent', sResults, 'pos', handles.pos, 'oGd', handles.oGd, 'class', handles.strClass);



% --- Executes on button press in pushbutton5.
end

function pushbutton5_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton5 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --- Executes on button press in uiOpen.
end

function uiOpen_Callback(hObject, eventdata, handles)
% hObject    handle to uiOpen (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --- Executes on button press in uiShowHide.
end

function uiShowHide_Callback(hObject, eventdata, handles)
% hObject    handle to uiShowHide (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of uiShowHide
if ~check_oGd(handles)
    set(hObject, 'Value', 0)
    return
end
if ~get(hObject, 'Value')
    draw_geom(handles.oGd);
    add_to_text(hObject, handles, '>> draw_geom(oGd)')
    return
end
if ~isfield(handles, 'pos')
    h = msgbox_no_button('Calculating location of sources and testing points...', 'SMTgui', 'help');
    make_geom(hObject, handles);
    handles = guidata(hObject);
    close(h);
end
pos = handles.pos;
add_to_text(hObject, handles, '>> show_pos(oGd, positions)')
show_pos(handles.oGd, pos);

% --- Executes on button press in uiDisp.
end

function uiDisp_Callback(hObject, eventdata, handles)
% hObject    handle to uiDisp (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


end

function make_geom(hObject, handles)
s = get(handles.text, 'string');
set(handles.text, 'string', strvcat(s, '>> positions = distribute(oGd);   % Distribute sources and testing points'));
handles.pos = distribute(handles.oGd);
guidata(hObject, handles);

end

function add_to_text(hObject, handles, varargin)
for ind = 1:nargin-2
    set(handles.text, 'string', strvcat(get(handles.text, 'string'), varargin{ind}))
end
height = 6;
s =  size(get(handles.text, 'string'),1);
if s > height
    set(handles.text, 'ListBoxTop', s-height);
end
pause(0.1)

end

function bOk = check_oGd(handles)
if ~isfield(handles, 'oGd')
    h = errordlg('Please load a geometry first.', 'Error');
    set(h, 'windowstyle', 'modal');
    waitfor(h);
    bOk = 0;
else
    bOk = 1;
end


% --- Executes on button press in uiSrc.
end

function uiSrc_Callback(hObject, eventdata, handles)
% hObject    handle to uiSrc (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
if ~check_oGd(handles)
    return
end
if isfield(handles, 'pos')
    data = gui_set_parameters('oGd', handles.oGd, 'pos', handles.pos);
else
    data = gui_set_parameters('oGd', handles.oGd);
end
if data.status == -1
    return
end
add_to_text(hObject, handles, ['>> oGd.nSources = ' num2str(data.oGd.nSources) ';'],...
    ['>> oGd.nTestingPoints = ' num2str(data.oGd.nTestingPoints) ';'],...
    ['>> oGd.alphaIn = ' num2str(data.oGd.alphaIn) '; % inner sources distance'],...
    ['>> oGd.alphaOut = ' num2str(data.oGd.alphaOut) '; % outer sources distance'])
handles.oGd = data.oGd;
handles.pos = data.pos;
oGd = data.oGd;
save([handles.gdPath handles.gdFile], 'oGd');
if get(handles.uiShowHide, 'Value') == 1
    show_pos(oGd, data.pos);
end
guidata(hObject, handles);


% --- Executes on button press in uiSave.
end

function uiSave_Callback(hObject, eventdata, handles)
global sResults
% hObject    handle to uiSave (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
if strcmp(handles.sesFile, 'untitled.se')
    uiSaveAs_Callback(hObject, eventdata, handles);
    if strcmp(handles.sesFile, 'untitled.se')
        return
    end
end
oSettings = handles;
fields = fieldnames(oSettings);
for ind = 1:length(fields)
    field = fields{ind};
    handle = eval(['handles.' field]);
    if all(ishandle(handle)) && handle ~= 0
        oSettings = rmfield(oSettings, field);
    end
end
save([handles.sesPath handles.sesFile], 'oSettings', 'sResults');
toggle_save(handles);

% --- Executes on button press in uiNew.
end

function uiNew_Callback(hObject, eventdata, handles)
global defPointsX defPointsY
global sResults sFirstResults
% hObject    handle to uiNew (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

bOk = check_before_close(handles);
if bOk
    if ~isSaveOff(handles)
        toggle_save(handles);
    end
else
    return
end
handles.maxNeffI = 0;
handles.output = hObject;
handles.min_lambda = 0;
handles.max_lambda = 0;
handles.n_lambda = 1;
handles.min_beta1 = 0;
handles.max_beta1 = 0;
handles.min_beta2 = 0;
handles.max_beta2 = 0;
handles.bPlot = 1;
handles.complex = -1;
handles.tol = 1e-6;
handles.absMinErr = 0.05;
handles.nSampMono = 7;
sFirstResults = 'Mode #    Eff. Index            CC Error';
sResults = sFirstResults;
handles.pointsx = defPointsX;
handles.pointsy = defPointsY;
handles.strClass = 'none';
if isfield(handles, 'oGd')
    handles = rmfield(handles, 'oGd');
end
if isfield(handles, 'pos')
    handles = rmfield(handles, 'pos');
end
set(handles.text, 'String', 'Command-line log');
set(handles.uiSmtgui, 'name', 'Smtgui - untitled.se (Geometry: None)');
handles.pointsx = defPointsX;
handles.pointsy = defPointsY;
handles.sComp = 'Ez';
handles.gdFile = 'None';
handles.gdPath = '';
handles.sesFile = 'untitled.se';
handles.method = 'direct';
cla(handles.uiAxes)
set(handles.uiAxes, 'visible', 'off');
set(handles.text, 'visible', 'on');
set(handles.uiPanel, 'visible', 'off');
% Update handles structure
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
end

function popupmenu2_CreateFcn(hObject, eventdata, handles)
% hObject    handle to popupmenu2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: popupmenu controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end


% --- Executes on selection change in popupmenu2.
end

function popupmenu2_Callback(hObject, eventdata, handles)
% hObject    handle to popupmenu2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: contents = get(hObject,'String') returns popupmenu2 contents as cell array
%        contents{get(hObject,'Value')} returns selected item from popupmenu2


% --- Executes on button press in pushbutton14.
end

function pushbutton14_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton14 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --- Executes on button press in pushbutton15.
end

function pushbutton15_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton15 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --- Executes on button press in togglebutton2.
end

function togglebutton2_Callback(hObject, eventdata, handles)
% hObject    handle to togglebutton2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of togglebutton2



% --- Executes on button press in uiSymmetry.
end

function uiSymmetry_Callback(hObject, eventdata, handles)
% hObject    handle to uiSymmetry (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
data = gui_Symmetry(handles.strClass);
if strcmp(handles.strClass, data.strClass)
    return
end
if isSaveOff(handles)
    toggle_save(handles)
end
add_to_text(hObject, handles, ['>> positions = cut_pos(oGd, positions, ''' data.strClass ''');']);
f = findobj('tag', 'uiSym');
if ~isempty(f)
    set(f, 'string', ['Class: ' data.strClass])
end
handles.strClass = data.strClass;
guidata(hObject, handles);
% Hint: get(hObject,'Value') returns toggle state of uiSymmetry


% --------------------------------------------------------------------
end

function new_session_Callback(hObject, eventdata, handles)
% hObject    handle to new_session (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
uiNew_Callback(hObject, eventdata, handles)

% --------------------------------------------------------------------
end

function uiSaveAs_Callback(hObject, eventdata, handles)
% hObject    handle to uiSaveAs (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
[filename, pathname, filter] = uiputfile( {'*.se','Session File (*.se)'},...
    'Save Session as:', 'untitled.se');
if filter < 1
    return
end
handles.sesFile = filename;
handles.sesPath = pathname;
set(handles.uiSmtgui, 'name', ['Smtgui - ' filename ' (Geometry: ' handles.gdFile ')']);
guidata(hObject, handles);
uiSave_Callback(hObject, eventdata, handles)

% --------------------------------------------------------------------
end

function save_session_Callback(hObject, eventdata, handles)
% hObject    handle to save_session (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
uiSave_Callback(hObject, eventdata, handles)



% --- Executes on button press in uiShowRes.
end

function uiShowRes_Callback(hObject, eventdata, handles)
% hObject    handle to uiShowRes (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
if get(hObject, 'Value')
    data = show_results('dummy', 'dummy', 'Toggle', hObject );
    handles.hResults = data.figure1;
    guidata(hObject, handles);
else
    f = findobj('name', 'Effective Indices');
    delete(f);
end




% --- Executes on button press in uiNewGeom.
end

function uiNewGeom_Callback(hObject, eventdata, handles)
% hObject    handle to uiNewGeom (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
data = gui_MakeCurves;
if ~isfield(data, 'oGd')
    return
end
add_to_text(hObject, handles, '>> % Spline curves stored in oGd.curveArray')
file_suggest = strip_ext(data.filename);
data = gui_SetEpsilon('oGd', data.oGd);
if data.status == -1
    return
end
add_to_text(hObject, handles, '>> % Sub-domains stored in oGd.sdArray',...
    '>> % permittivities stored in oGd.sdArray.Er',...
    '>> % first sub-domain is always the background material')
data = gui_set_parameters('oGd', data.oGd);
if data.status == -1
    return
end
add_to_text(hObject, handles, ['>> oGd.nSources = ' num2str(data.oGd.nSources) ';'],...
    ['>> oGd.nTestingPoints = ' num2str(data.oGd.nTestingPoints) ';'],...
    ['>> oGd.alphaIn = ' num2str(data.oGd.alphaIn) '; % inner sources distance'],...
    ['>> oGd.alphaOut = ' num2str(data.oGd.alphaOut) '; % outer sources distance'])
[filename, pathname, filter] = uiputfile({'*.gd', 'Geometry Description Files (*.gd)'},...
    'Save As...', [file_suggest 'gd']);
if filter < 1
    return
end
oGd = data.oGd;
save([pathname filename], 'oGd');



% --- Executes on button press in uiEpsilon.
end

function uiEpsilon_Callback(hObject, eventdata, handles)
% hObject    handle to uiEpsilon (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of uiEpsilon
if ~check_oGd(handles)
    return
end
data = gui_SetEpsilon('oGd', handles.oGd);
if data.status == -1
    return
end
add_to_text(hObject, handles, '>> % Sub-domains stored in oGd.sdArray',...
    '>> % permittivities stored in oGd.sdArray.Er',...
    '>> % first sub-domain is always the background material')
handles.oGd = data.oGd;
oGd = handles.oGd;
save([handles.gdPath handles.gdFile], 'oGd')
guidata(hObject, handles);



% --------------------------------------------------------------------
end

function uiMenuNewGeom_Callback(hObject, eventdata, handles)
% hObject    handle to uiMenuNewGeom (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
uiNewGeom_Callback(handles.uiNewGeom, eventdata, handles)



% --------------------------------------------------------------------
end

function uiDXF_Callback(hObject, eventdata, handles)
% hObject    handle to uiDXF (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
if ~check_oGd(handles)
    return
end
sSuggest = [strip_ext(handles.gdFile) 'dxf'];
[FileName, PathName, FilterIndex] = uiputfile('DXF File (*.dxf)', 'Save as', sSuggest);
if FilterIndex < 1
    return
end
fn = [PathName FileName];
pause(0.3)
h = msgbox_no_button('Exporting...', 'SMTgui', 'help');
pause(0.1)
ogd2dxf(handles.oGd, fn);
uiwait(msgbox('Exporting... Done.', 'SMTgui', 'help', 'replace'))
% --------------------------------------------------------------------
end

function uiSet_Callback(hObject, eventdata, handles)
% hObject    handle to uiSet (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
end

function uiMenuEpsilon_Callback(hObject, eventdata, handles)
% hObject    handle to uiMenuEpsilon (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
uiEpsilon_Callback(handles.uiEpsilon, eventdata, handles)

% --------------------------------------------------------------------
end

function uiMenuSrc_Callback(hObject, eventdata, handles)
% hObject    handle to uiMenuSrc (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
uiSrc_Callback(handles.uiSrc, eventdata, handles)



% --------------------------------------------------------------------
end

function uiMenuSym_Callback(hObject, eventdata, handles)
% hObject    handle to uiMenuSym (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
uiSymmetry_Callback(handles.uiSymmetry, eventdata, handles)



% --------------------------------------------------------------------
end

function uiMenuFind_Callback(hObject, eventdata, handles)
% hObject    handle to uiMenuFind (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
uiFindModes_Callback(handles.uiFindModes, eventdata, handles)

% --------------------------------------------------------------------
end

function uiSolve_Callback(hObject, eventdata, handles)
% hObject    handle to uiSolve (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)




% --------------------------------------------------------------------
end

function uiMenuPlot_Callback(hObject, eventdata, handles)
% hObject    handle to uiMenuPlot (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
uiPlotFields_Callback(handles.uiPlotFields, eventdata, handles)

end

function toggle_save(handles)
state = get(handles.uiSave, 'Enable');
if ~isSaveOff(handles)
    set(handles.uiSave, 'enable', 'off')
    set(handles.save_session, 'enable', 'off')
else
    set(handles.uiSave, 'enable', 'on')
    set(handles.save_session, 'enable', 'on')
end

end

function out = isSaveOff(handles)
out = strcmp(lower(get(handles.uiSave, 'Enable')), 'off');

end

function bOk = check_before_close(handles)
if ~isSaveOff(handles)
    selection = questdlg([handles.sesFile ' session is not saved. Close without saving?'],...
        ['Close Session...'],...
        'Yes','No','Yes');
    if strcmp(selection,'No')
        bOk = 0;
        return;
    end
end
bOk = 1;


% --- Executes when user attempts to close uiSmtgui.
end

function uiSmtgui_CloseRequestFcn(hObject, eventdata, handles)
% hObject    handle to uiSmtgui (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: delete(hObject) closes the figure
CloseMenuItem_Callback(hObject, eventdata, handles)




% --------------------------------------------------------------------
end

function uiSaveLog_Callback(hObject, eventdata, handles)
% hObject    handle to uiSaveLog (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

s = get(handles.text, 'string');
[FileName, PathName, FilterIndex] = uiputfile('Text File (*.txt)', 'Save as', 'smt_log.txt');
if FilterIndex < 1
    return
end
fn = [PathName FileName];
fid = fopen(fn, 'w');
for ind = 1:size(s, 1)
    fprintf(fid, s(ind,:));
    fprintf(fid, '\n');
end
fclose(fid);
% --------------------------------------------------------------------
end

function uiClearLog_Callback(hObject, eventdata, handles)
% hObject    handle to uiClearLog (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
set(handles.text, 'string', 'Command-line log')

% --------------------------------------------------------------------
end
function uiLog_Callback(hObject, eventdata, handles)
% hObject    handle to uiLog (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --- Executes on button press in uiCancelSearch.
end
function uiCancelSearch_Callback(hObject, eventdata, handles)
% hObject    handle to uiCancelSearch (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --- Executes on button press in uiJump.
end
function uiJump_Callback(hObject, eventdata, handles)
% hObject    handle to uiJump (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end


% --------------------------------------------------------------------
function uiAbout_Callback(hObject, eventdata, handles)
% hObject    handle to uiAbout (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
gui_About
end



% --------------------------------------------------------------------
function uiGuide_Callback(hObject, eventdata, handles)
% hObject    handle to uiGuide (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
winopen('smtp_help.html')
end
% --------------------------------------------------------------------
function Untitled_1_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
end

